# -*- coding: utf-8 -*-

"""Multimediawidgets module."""

from .videowidgetcontrol import VideoWidgetControl
from .videowidget import VideoWidget

__all__ = [
    "VideoWidgetControl",
    "VideoWidget",
]
